@@ -19,18 +19,22 @@ A module of statistical point estimators and likelihood functions.
 
 from os import path
 from sys import path as sys_path
-from typing import List
-from numpy import log, sqrt, exp, inf
+from typing import List, Union
+from numpy import inf
+from math import log, sqrt, exp
 from scipy.stats import norm, multivariate_normal
 
+from scrilla.util import errors
+
 if __name__ == "__main__":
     APP_DIR = path.dirname(path.dirname(path.abspath(__file__)))
     sys_path.append(APP_DIR)
 
-from scrilla import settings, errors, cache
+from scrilla import settings, cache
+from scrilla.static.constants import constants
 import scrilla.util.outputter as outputter
 
-logger = outputter.Logger('estimators', settings.LOG_LEVEL)
+logger = outputter.Logger('scrilla.analysis.estimators', settings.LOG_LEVEL)
 profile_cache = cache.ProfileCache()
 correlation_cache = cache.CorrelationCache()
 
@@ -71,7 +75,7 @@ def bivariate_normal_likelihood_function(params: list, data: list) -> float:
     cov = [[params[2], params[4]], [params[4], params[3]]]
 
     determinant = params[2]*params[3] - params[4]**2
-    if determinant == 0 or determinant < 0 or determinant < 0.00000001:
+    if determinant == 0 or determinant < 0 or determinant < (10**(-constants['ACCURACY'])):
         return inf
 
     likelihood = 0
@@ -80,7 +84,7 @@ def bivariate_normal_likelihood_function(params: list, data: list) -> float:
     return likelihood
 
 
-def sample_percentile(data: list, percentile: float):
+def sample_percentile(data: List[float], percentile: float):
     """
     Returns the observation in a sample data corresponding to the given percentile, i.e. the observation from a sorted sample where the percentage of the observations below that point is specified by the percentile. If the percentile falls between data points, the observation is smoothed based on the distance from the adjoining observations in the following manner,
 
@@ -108,7 +112,32 @@ def sample_percentile(data: list, percentile: float):
     return (1-weight)*data[first_index] + weight*data[second_index]
 
 
-def sample_correlation(x: list, y: list):
+def empirical_copula(sample: List[List[float]], x_order: float, y_order: float):
+    """
+    Computes an empirical estimate of the copula distribution for a bivariate sample, i.e.
+
+    $$ C(u_{1}, u_{2}) = F_{c} ({U_{1}) < F_1^{-1}(u_{1}), {U_{2}}< F_2^{-1}(u_{2})) $$
+
+    Using the empirical estimate defined by,
+
+    $$ C_{n} = \frac{\{ (x,y) \vert x \leq x_{p} & y \leq y_{p} \}}{n} $$ 
+
+    where \\(x_p\\) and \\(y_p\\) are the *p*-th percentiles of their respective univariate samples.
+    """
+    n = len(sample)
+
+    def x_order_bounds(test_point):
+        return test_point < x_order or test_point == x_order
+
+    def y_order_bounds(test_point):
+        return test_point < y_order or test_point == y_order
+
+    copula = [1 for point in sample if x_order_bounds(
+        point[0]) and y_order_bounds(point[1])]
+    return len(copula) / n
+
+
+def sample_correlation(x: List[float], y: List[float]):
     """
     Returns the sample correlation calculated using the Pearson correlation coefficient estimator,
 
@@ -123,9 +152,9 @@ def sample_correlation(x: list, y: list):
 
     Raises 
     ------
-    1. **scrilla.errors.SampleSizeError**
+    1. `scrilla.errors.SampleSizeError` :
         If the sample sizes do not meet the requirements for estimation, this error will be thrown.
-    2. **ValueError**
+    2. **ValueError** :
         If the denominator of the correlation coefficient becomes too small for floating point arithmetic, this error is thrown.
 
     .. todos ::
@@ -171,11 +200,11 @@ def sample_correlation(x: list, y: list):
 
 def recursive_rolling_correlation(correl_previous, new_x_observation, lost_x_obs,
                                   new_y_obs, lost_y_obs, n=settings.DEFAULT_ANALYSIS_PERIOD):
-
+    # TODO: after other rolling functions work...
     pass
 
 
-def sample_mean(x: list) -> float:
+def sample_mean(x: List[float]) -> float:
     r"""
     Returns the sample mean from a sample of data \\(\{x_1 , x_2, ... , x_n \}\\),
 
@@ -183,16 +212,22 @@ def sample_mean(x: list) -> float:
 
     Parameters
     ----------
-    1. **x**: ``list``
+    1. **x**: ``List[Union[float,int]]``
         List containing a sample of numerical data.
 
     Raises 
     ------
     1. **scrilla.errors.SampleSizeError**
-        If `len(x)==0`, this error will be thrown.
+        If ``len(x)==0``, this error will be thrown.
+    2. **ValueError**
+        If the sample contains null or non-numerical data, this error will be thrown.
     """
     xbar, n = 0, len(x)
 
+    if not all(this_x is not None and isinstance(this_x, (float, int)) for this_x in x):
+        raise ValueError(
+            'Sample contains null values')
+
     if n == 0:
         raise errors.SampleSizeError(
             'Sample mean cannot be computed for a sample size of 0.')
@@ -207,7 +242,7 @@ def recursive_rolling_mean(xbar_previous, new_obs, lost_obs, n=settings.DEFAULT_
     return xbar_next
 
 
-def sample_variance(x: list):
+def sample_variance(x: List[float]):
     r"""
     Returns the sample variance from a sample of data \\(\{x_1 , x_2, ... , x_n \}\\),
 
@@ -220,11 +255,16 @@ def sample_variance(x: list):
 
     Raises 
     ------
-    1. **scrilla.errors.SampleSizeError**
+    1. `scrilla.errors.SampleSizeError`
     """
+    # TODO: this is a 'naive' estimation of variance: https://en.wikipedia.org/wiki/Algorithms_for_calculating_variance
 
     mu, sigma, n = sample_mean(x=x), 0, len(x)
 
+    if not all(this_x is not None and isinstance(this_x, (float, int)) for this_x in x):
+        raise ValueError(
+            'Sample contains null values')
+
     if n in [0, 1]:
         raise errors.SampleSizeError(
             'Sample variance cannot be computed for a sample size less than or equal to 1.')
@@ -235,6 +275,7 @@ def sample_variance(x: list):
 
 
 def recursive_rolling_variance(var_previous, xbar_previous, new_obs, lost_obs, n=settings.DEFAULT_ANALYSIS_PERIOD):
+    # TODO: slightly off for some reason...Formula not correct? Rework it out.
     xbar_new = recursive_rolling_mean(xbar_previous=xbar_previous, new_obs=new_obs,
                                       lost_obs=lost_obs, n=n)
     var_new = var_previous + \
@@ -253,8 +294,8 @@ def sample_covariance(x: list, y: list):
 
     Raises 
     ------
-    1. **scrilla.errors.SampleSizeError**
-        If `len(x) != len(y)` (samples of incomparable length) or `len(x) in [0,1]` (insufficient data/degrees of freedom), this error will be thrown.
+    1. `scrilla.errors.SampleSizeError`
+        If ``len(x) != len(y)`` (samples of incomparable length) or ``len(x) in [0,1]`` (insufficient data/degrees of freedom), this error will be thrown.
     """
 
     if len(x) != len(y):
@@ -274,8 +315,8 @@ def sample_covariance(x: list, y: list):
     return covariance
 
 
-def recursive_rolling_covariance(covar_previous, new_x_obs, lost_x_obs, previous_x_bar,
-                                 new_y_obs, lost_y_obs, previous_y_bar, n=settings.DEFAULT_ANALYSIS_PERIOD):
+def recursive_rolling_covariance(covar_previous: float, new_x_obs: float, lost_x_obs: float, previous_x_bar: float, new_y_obs: float, lost_y_obs: float, previous_y_bar: float, n: int = settings.DEFAULT_ANALYSIS_PERIOD):
+    # TODO: no work.
     new_sum_term = new_x_obs*new_y_obs - lost_x_obs*lost_y_obs
     xy_cross_term = previous_x_bar*(new_y_obs-lost_y_obs)
     yx_cross_term = previous_y_bar*(new_x_obs-lost_x_obs)
@@ -285,7 +326,7 @@ def recursive_rolling_covariance(covar_previous, new_x_obs, lost_x_obs, previous
     return covar_new
 
 
-def simple_regression_beta(x: list, y: list):
+def simple_regression_beta(x: List[float], y: List[float]):
     """
     Parameters
     ----------
@@ -296,8 +337,8 @@ def simple_regression_beta(x: list, y: list):
 
     Raises 
     ------
-    1. **scrilla.errors.statistics.SampleSizeError**
-        If `len(x) != len(y)` (samples of incomparable length) or `len(x) < 3` (insufficient data/degrees of freedom), this error will be thrown.
+    1. `scrilla.errors.statistics.SampleSizeError`
+        If ``len(x) != len(y)`` (samples of incomparable length) or ``len(x) < 3`` (insufficient data/degrees of freedom), this error will be thrown.
     """
 
     if len(x) != len(y):
@@ -314,7 +355,7 @@ def simple_regression_beta(x: list, y: list):
     return beta
 
 
-def simple_regression_alpha(x: list, y: list):
+def simple_regression_alpha(x: List[float], y: List[float]):
     """
     Parameters
     ----------
@@ -325,8 +366,8 @@ def simple_regression_alpha(x: list, y: list):
 
     Raises 
     ------
-    1. **scrilla.errors.SampleSizeError**
-        If `len(x) != len(y)` (samples of incomparable length) or `len(x) < 3` (insufficient data/degrees of freedom), this error will be thrown.
+    1. `scrilla.errors.SampleSizeError`
+        If ``len(x) != len(y)`` (samples of incomparable length) or ``len(x) < 3`` (insufficient data/degrees of freedom), this error will be thrown.
     """
 
     if len(x) != len(y):
@@ -343,7 +384,7 @@ def simple_regression_alpha(x: list, y: list):
     return alpha
 
 
-def qq_series_for_sample(sample: list) -> List[list]:
+def qq_series_for_sample(sample: List[float]) -> List[list]:
     """
     Calculates the QQ series for a sample of data, i.e. the set defined by the ordered pair of sample percentiles and theoretical normal percentiles. A sample's normality can be assessed by how linear the result graph is.
 
@@ -354,7 +395,7 @@ def qq_series_for_sample(sample: list) -> List[list]:
     """
     qq_series = []
     n = len(sample)
-    for i, point in enumerate(sample):
+    for i in range(len(sample)):
         percentile = (i + 0.5)/n
         percentile_sample = sample_percentile(
             data=sample, percentile=percentile)
@@ -362,3 +403,9 @@ def qq_series_for_sample(sample: list) -> List[list]:
         qq_series += [[percentile_norm, percentile_sample]]
 
     return qq_series
+
+
+def standardize(x: List[float]):
+    mu = sample_mean(x)
+    sigma = sqrt(sample_variance(x))
+    return [(this_x - mu)/sigma for this_x in x]
