@@ -22,10 +22,11 @@ from scipy.integrate import quad
 from numpy import isinf, inf
 import random
 
-from scrilla import settings, errors
+from scrilla import settings
+from scrilla.util import errors
 import scrilla.util.outputter as outputter
 
-logger = outputter.Logger("calculator", settings.LOG_LEVEL)
+logger = outputter.Logger("scrilla.analysis.integration", settings.LOG_LEVEL)
 
 
 def generate_random_walk(periods: int):
