@@ -12,15 +12,18 @@
 # You should have received a copy of the GNU General Public License
 # along with scrilla.  If not, see <https://www.gnu.org/licenses/>
 # or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
-
-from scipy.stats import norm
-from numpy import log, sqrt, exp
-
 """
 This module contains probability functions unique to the Geometric Brownian Motion stochastic model. All functions assume an asset price process that follows GBM. In other words, the return distribution is lognormal. This is equivalent to the Black-Scholes model from finance. 
 
 The calculations have no stance on whether the actual probability distribution or the risk-neutral probability distribution should be used. This is implicitly specified by the user when they provided a rate of return to functions in this module. If the user wishes to use a risk-neutral probability, simply provide the assumed risk-free rate. If the user wishes to use an actual probability, provide the estimate of the rate of the return for the asset.
 """
+from scipy.stats import norm
+from math import sqrt, exp, log
+from scrilla import settings
+from scrilla.util.outputter import Logger
+
+logger = Logger('scrilla.analysis.models.geometric.statistics',
+                settings.LOG_LEVEL)
 
 
 def d1(S0: float, ST: float, vol: float, ret: float, expiry: float, div: float = 0) -> float:
@@ -104,6 +107,8 @@ def prob_d1(S0: float, ST: float, vol: float, ret: float, expiry: float, div: fl
     -------
     ``float`` : cumulative probability of *d1*
     """
+    if vol == 0:
+        pass
     thisD1 = d1(S0=S0, ST=ST, vol=vol, ret=ret, expiry=expiry, div=div)
     if neg:
         thisD1 = -thisD1
@@ -135,13 +140,22 @@ def prob_d2(S0: float, ST: float, vol: float, ret: float, expiry: float, div: fl
     -------
     ``float`` : cumulative probability of *d2*
     """
+    # if no uncertainty
+    if vol == 0:
+        # if the future value of initial price is greater than the strike
+        if S0*exp((ret-div)*expiry) > ST:
+            # probability(S>St)=1
+            return 1
+        # probability(S>St)=0
+        return 0
+
     thisD2 = d2(S0=S0, ST=ST, vol=vol, ret=ret, expiry=expiry, div=div)
     if neg:
         thisD2 = -thisD2
     return norm.cdf(thisD2)
 
 
-def percentile(S0: float, vol: float, ret: float, expiry: float, percentile: float, div: float = 0) -> float:
+def percentile(S0: float, vol: float, ret: float, expiry: float, prob: float, div: float = 0) -> float:
     """
     Returns the final value of the asset price over the time horizon `expiry` that corresponds the `percentile` of the price's distribution. The asset price process is assumed to follow Geometric Brownian Motion, i.e. the return distribution is lognormal. 
 
@@ -155,8 +169,11 @@ def percentile(S0: float, vol: float, ret: float, expiry: float, percentile: flo
         Annualized return of the asset price process.
     4. **expiry**: ``float``
         Time horizon in years, i.e. the time delta between `ST` and `S0` measured in years. 
-    5. **percentile**: ``float``
-        Percentile of the distribution to be returned, i.e. if S is the price process, Pr(S<St)=percentile.
+    5. **prob**: ``float``
+        Percentile of the distribution to be returned, i.e. if S is the price process and \\(S_t\\) is the percentile,
+
+        $$ Pr(S<S_t)=prob $$
+
     6. **div**: ``float``
         *Optional*. Annualized dividend yield. Defaults to 0. 
 
@@ -167,7 +184,7 @@ def percentile(S0: float, vol: float, ret: float, expiry: float, percentile: flo
     .. notes::
         * Percentiles are preserved under log transformations, so the asset percentile is equal to the exponentiated return percentile. See Wiki article for more information: https://en.wikipedia.org/wiki/Log-normal_distribution#Mode,_median,_quantiles
     """
-    inv_norm = norm.ppf(percentile)
+    inv_norm = norm.ppf(prob)
     exponent = (ret - div - 0.5*(vol**2))*expiry + vol*sqrt(expiry)*inv_norm
     return (S0*exp(exponent))
 
