@@ -15,12 +15,13 @@
 
 import datetime
 
-from scrilla import services, settings, errors
+from scrilla import services, settings
 from scrilla.static import constants
-from scrilla.util import dater, outputter
+from scrilla.util import dater, errors, outputter
 import scrilla.analysis.estimators as estimators
 
-logger = outputter.Logger('analysis.objects.cashflow', settings.LOG_LEVEL)
+logger = outputter.Logger(
+    'scrilla.analysis.objects.cashflow', settings.LOG_LEVEL)
 
 # Technically these are periods
 FREQ_DAY = 1/365
@@ -82,8 +83,9 @@ class Cashflow:
 
         # if constant is specified, override sample and growth_function
         if constant is not None:
-            logger.debug(f'constant = $ {constant}; period MUST NOT be null!')
-            logger.debug(f'period = {self.period}')
+            logger.debug(
+                f'constant = $ {constant}; period MUST NOT be null!', 'Cashflow.__init__')
+            logger.debug(f'period = {self.period}', 'Cashflow.__init__')
             self.constant = constant
             self.sample = None
             self.growth_function = None
@@ -100,7 +102,8 @@ class Cashflow:
         else:
             self.discount_rate = discount_rate
 
-        logger.debug(f'Using discount_rate = {self.discount_rate}')
+        logger.debug(
+            f'Using discount_rate = {self.discount_rate}', 'Cashflow.__init__')
 
         # If no frequency is specified, infer frequency from sample
         if self.sample is not None and self.period is None:
@@ -110,7 +113,8 @@ class Cashflow:
             self.time_to_today = self.calculate_time_to_today()
 
     def infer_period(self):
-        logger.debug('Attempting to infer period/frequency of cashflows.')
+        logger.debug(
+            'Attempting to infer period/frequency of cashflows.', 'Cashflow.infer_period')
 
         # no_of_dates = len - 1 because delta is being computed, i.e.
         #   lose one date.
@@ -120,26 +124,29 @@ class Cashflow:
 
         if no_of_dates < 2:
             logger.debug(
-                'Cannot infer period from sample size less than or equal to 1')
+                'Cannot infer period from sample size less than or equal to 1', 'Cashflow.infer_period')
             self.period = None
             self.frequency = None
 
         else:
             for date in dates:
                 if first_pass:
-                    tomorrows_date = dater.parse_date_string(date)
+                    tomorrows_date = dater.parse(date)
                     first_pass = False
 
                 else:
-                    todays_date = dater.parse_date_string(date)
+                    todays_date = dater.parse(date)
+                    # TODO: 365 or 252?
                     delta = (tomorrows_date - todays_date).days / 365
                     mean_delta += delta / no_of_dates
                     tomorrows_date = todays_date
 
             self.period = mean_delta
             self.frequency = 1 / self.period
-            logger.debug(f'Inferred period = {self.period} yrs')
-            logger.debug(f'Inferred frequency = {self.frequency}')
+            logger.debug(
+                f'Inferred period = {self.period} yrs', 'Cashflow.infer_period')
+            logger.debug(
+                f'Inferred frequency = {self.frequency}', 'Cashflow.infer_period')
 
     # TODO: trading days or actual days?
     def generate_time_series_for_sample(self):
@@ -149,20 +156,20 @@ class Cashflow:
 
         if no_of_dates == 0:
             logger.debug(
-                'Cannot generate a time series for a sample size of 0.')
+                'Cannot generate a time series for a sample size of 0.', 'Cashflow.generate_time_series_for_sample')
             self.time_series = None
         else:
-            first_date = dater.parse_date_string(list(dates)[-1])
+            first_date = dater.parse(list(dates)[-1])
 
             for date in dates:
-                this_date = dater.parse_date_string(date)
+                this_date = dater.parse(date)
                 delta = (this_date - first_date).days
                 time_in_years = delta / 365
                 self.time_series.append(time_in_years)
 
     # TODO: trading days or actual days?
     def calculate_time_to_today(self):
-        first_date = dater.parse_date_string(list(self.sample.keys())[-1])
+        first_date = dater.parse(list(self.sample.keys())[-1])
         today = datetime.date.today()
         return ((today - first_date).days/365)
 
@@ -178,15 +185,16 @@ class Cashflow:
             if len(self.sample) > 0:
                 self.alpha = list(self.sample.items())[0][1]
                 logger.debug(
-                    'Error calculating regression coefficients; Defaulting to Markovian process E(X2|X1) = X1.')
-                logger.debug(f'Estimation model : y = {self.alpha}')
+                    'Error calculating regression coefficients; Defaulting to Markovian process E(X2|X1) = X1.', 'Cashflow.regress_growth_function')
+                logger.debug(
+                    f'Estimation model : y = {self.alpha}', 'regress_growth_function')
             else:
                 raise errors.SampleSizeError(
                     'Not enough information to formulate estimation model.')
 
         else:
             logger.debug(
-                f'Linear regression model : y = {self.beta} * x + {self.alpha}')
+                f'Linear regression model : y = {self.beta} * x + {self.alpha}', 'Cashflow.regress_growth_function')
 
     def generate_model_series(self):
         return [self.alpha + self.beta*time for time in self.time_series]
@@ -236,6 +244,10 @@ class Cashflow:
         -------
         ``float`` : NPV of cash flow.
         """
+        if self.discount_rate < 0:
+            raise errors.ModelError(
+                f'Model assumptions violated: Cannot a future value with a discount rate of {self.discount_rate}')
+
         if self.period is None:
             raise errors.InputValidationError(
                 "No period detected for cashflows. Not enough information to calculate net present value.")
@@ -258,7 +270,7 @@ class Cashflow:
 
         else:
             dates = self.sample.keys()
-            latest_date = dater.parse_date_string(list(dates)[0])
+            latest_date = dater.parse(list(dates)[0])
             time_to_first_payment = dater.get_time_to_next_period(
                 starting_date=latest_date, period=self.period)
 
@@ -267,7 +279,6 @@ class Cashflow:
         while calculating:
             previous_value = net_present_value
             current_time = time_to_first_payment + i * self.period
-
             net_present_value += self.get_growth_function(current_time) / (
                 (1 + self.discount_rate)**current_time)
 
