@@ -13,18 +13,26 @@
 # along with scrilla.  If not, see <https://www.gnu.org/licenses/>
 # or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
 
-import numpy
-import math
+from datetime import date
+from math import trunc, sqrt
 from decimal import Decimal
+from itertools import groupby
+from typing import Callable, Dict, List, Union
 
-from scrilla import services, settings
-import scrilla.util.outputter as outputter
+# TODO: get rid of numpy functions.
+#       dot, multiply and transpose should be easy to replicate
+#       and it removes a big dependency from the package...
+from numpy import dot, multiply, transpose
 
+from scrilla import settings
+from scrilla.static import keys
+from scrilla.util import errors, outputter
 # TODO: conditional import module based on analysis_mode, i.e. geometric versus mean reverting.
-import scrilla.analysis.models.geometric.statistics as statistics
-import scrilla.analysis.models.geometric.probability as probability
+from scrilla.analysis.models.geometric.statistics import calculate_risk_return, correlation_matrix
+from scrilla.analysis.models.geometric.probability import percentile, conditional_expected_value
 
-logger = outputter.Logger("analysis.objects.portfolio", settings.LOG_LEVEL)
+logger = outputter.Logger(
+    "scrilla.analysis.objects.portfolio", settings.LOG_LEVEL)
 
 # TODO: allow user to specify bounds for equities, i.e. min and max allocations.
 
@@ -39,21 +47,21 @@ class Portfolio:
 
     Parameters
     ----------
-    1. **tickers**: ``list``
+    1. **tickers**: ``List[str]``
         An array of ticker symbols that decribe the assets in a portfolio.
-    2. **start_date**: ``datetime.date``
+    2. **start_date**: ``Union[date, None]``
         *Optional*. The start date for the range of historical prices over which the portfolio will be optimized. 
-    3. **end_date**: ``datetime.date``
+    3. **end_date**: ``Union[date, None]``
         *Optional*. The end date for the range of historical prices over which the portfolio will be optimized.
-    4. **sample_prices**: ``dict``
+    4. **sample_prices**: ``Union[Dict[str, Dict[str, float]], None]``
         *Optional*. A list representing a sample of historical data over a time range. The list must be ordered in descending order, i.e. from latest to earliest. Must be formatted as: `{ 'ticker_1': { 'date' : { 'open': value, 'close': value},... }}
-    5. **risk_profile** : ``dict``
+    5. **risk_profile** : ``Union[Dict[str, Dict[str, float]], None]``
         Optional: Rather than use sample statistics calculated from historical data, this argument can override the calculated values. Must be formatted as: `{ ticker: { 'annual_return': float, 'annual_volatility': float }}`
-    6. **correlation_matrix**: ``[ list ]``
+    6. **correlation_matrix**: ``Union[List[List[float]], None]``
         Optional: Rather than use correlations calculated from historical data, this argument can override the calculated vlaues.
-    7. **asset_return_functions**: ``[ function(t) ]``
+    7. **asset_return_functions**: ``Union[List[Callable], None]``
         *Optional*. An array of function that describes the expected logarithmic rate of return of each asset in the portfolio with respect to time. The order between `asset_return_functions` and `tickers` be must be preserved, i.e. the index of tickers must correspond to the symbol described by the function with same index in `asset_return_functions`.
-    8. **asset_volatility_funtions**: ``[ function(t) ]``
+    8. **asset_volatility_funtions**: ``Union[List[Callable], None]``
         *Optional*. An array of functions that describe the mean volatility of each asset in the portfolio with respect to time. The order between `asset_volatility_functions` and `tickers` be must be preserved, i.e. the index of tickers must correspond to the symbol described by the function with the same index in `asset_volatility_functions`.
 
     Attributes
@@ -73,41 +81,54 @@ class Portfolio:
     where B(t) ~ \\(N(0, \Delta \cdot t)\\).
     """
 
-    def __init__(self, tickers, start_date=None, end_date=None, sample_prices=None,
-                 correlation_matrix=None, risk_profiles=None, risk_free_rate=None,
-                 asset_return_functions=None, asset_volatility_functions=None,
-                 method=settings.ESTIMATION_METHOD):
-        self.shares = None
-        self.actual_total = None
-        self.risk_free_rate = None
+    def __init__(self, tickers: List[str], start_date=Union[date, None], end_date=Union[date, None], sample_prices: Union[Dict[str, Dict[str, float]], None] = None, correl_matrix: Union[List[List[int]], None] = None, risk_profiles: Union[Dict[str, Dict[str, float]]] = None, risk_free_rate: Union[float, None] = None, asset_return_functions: Union[List[Callable], None] = None, asset_volatility_functions: Union[List[Callable], None] = None, method: str = settings.ESTIMATION_METHOD):
         self.estimation_method = method
-
-        if sample_prices is None:
-            self.start_date = start_date
-            self.end_date = end_date
-        else:
-            self.start_date = list(sample_prices.keys())[-1]
-            self.end_date = list(sample_prices.keys())[0]
-
-        self.tickers = tickers
         self.sample_prices = sample_prices
-        self.correlation_matrix = correlation_matrix
+        self.tickers = tickers
+        self.correl_matrix = correl_matrix
         self.asset_volatility_functions = asset_volatility_functions
         self.asset_return_functions = asset_return_functions
         self.risk_profiles = risk_profiles
         self.target_return = None
 
+        if self.sample_prices is None:
+            self.start_date = start_date
+            self.end_date = end_date
+        else:
+            self.start_date = list(self.sample_prices.keys())[-1]
+            self.end_date = list(self.sample_prices.keys())[0]
+
         if risk_free_rate is not None:
             self.risk_free_rate = risk_free_rate
         else:
-            self.risk_free_rate = services.get_risk_free_rate()
+            self.risk_free_rate = 0
 
-        self.calculate_stats()
+        self._init_asset_types()
+        self._init_dates()
+        self._init_stats()
 
-        # todo: calculate stats with lambda functions.
-    # Returns False if calculations fail
+    def _init_asset_types(self):
+        self.asset_types = []
+        for ticker in self.tickers:
+            self.asset_types.append(errors.validate_asset_type(ticker))
 
-    def calculate_stats(self):
+        self.asset_groups = 0
+        for _ in groupby(sorted(self.asset_types)):
+            self.asset_groups += 1
+
+    def _init_dates(self):
+        if self.asset_groups == 1 and self.asset_types[0] == keys.keys['ASSETS']['CRYPTO']:
+            self.start_date, self.end_date = errors.validate_dates(self.start_date,
+                                                                   self.end_date,
+                                                                   keys.keys['ASSETS']['CRYPTO'])
+            self.weekends = 1
+        else:
+            self.start_date, self.end_date = errors.validate_dates(self.start_date,
+                                                                   self.end_date,
+                                                                   keys.keys['ASSETS']['EQUITY'])
+            self.weekends = 0
+
+    def _init_stats(self):
         self.mean_return = []
         self.sample_vol = []
 
@@ -123,17 +144,29 @@ class Portfolio:
 
         else:
 
+            # TODO: there is a logical error here. if the portfolio is made up of mixed assets (crypto, equity),
+            #       then calculate_risk_return will calculate the risk profile over a different time period than
+            #       the correlation matrix. the reason is: risk_return is univariate, but correlation is bivariate,
+            #       so when the correlation of an equity and crypto is calculated, it truncates the sample to dates
+            #       where both assets trade, i.e. crypto prices on weekends get ignored. the risk_profile of the crypto
+            #       will be over a shorter date range because the analysis will include weekends, whereas the crypto
+            #       correlation will not include weekends if the asset types are mixed. the problem is further
+            #       compounded since the correlation method will retrieve the univariate profile to use in its calculation.
+            #       need a flag in the cache to tell the program the statistic includes/exclude weekend prices.
+
             if self.risk_profiles is None:
                 for ticker in self.tickers:
                     if self.sample_prices is not None:
-                        stats = statistics.calculate_risk_return(ticker=ticker,
-                                                                 sample_prices=self.sample_prices[ticker],
-                                                                 method=self.estimation_method)
+                        stats = calculate_risk_return(ticker=ticker,
+                                                      sample_prices=self.sample_prices[ticker],
+                                                      method=self.estimation_method,
+                                                      weekends=self.weekends)
                     else:
-                        stats = statistics.calculate_risk_return(ticker=ticker,
-                                                                 start_date=self.start_date,
-                                                                 end_date=self.end_date,
-                                                                 method=self.estimation_method)
+                        stats = calculate_risk_return(ticker=ticker,
+                                                      start_date=self.start_date,
+                                                      end_date=self.end_date,
+                                                      method=self.estimation_method,
+                                                      weekends=self.weekends)
 
                     self.mean_return.append(stats['annual_return'])
                     self.sample_vol.append(stats['annual_volatility'])
@@ -144,12 +177,13 @@ class Portfolio:
                     self.sample_vol.append(
                         self.risk_profiles[ticker]['annual_volatility'])
 
-            if self.correlation_matrix is None:
-                self.correlation_matrix = statistics.correlation_matrix(tickers=self.tickers,
-                                                                        start_date=self.start_date,
-                                                                        end_date=self.end_date,
-                                                                        sample_prices=self.sample_prices,
-                                                                        method=self.estimation_method)
+            if self.correl_matrix is None:
+                self.correl_matrix = correlation_matrix(tickers=self.tickers,
+                                                        start_date=self.start_date,
+                                                        end_date=self.end_date,
+                                                        sample_prices=self.sample_prices,
+                                                        weekends=self.weekends,
+                                                        method=self.estimation_method)
 
     def return_function(self, x):
         """
@@ -165,7 +199,7 @@ class Portfolio:
         ``float``
             The portfolio return on an annualized basis.
         """
-        return numpy.dot(x, self.mean_return)
+        return dot(x, self.mean_return)
 
     def volatility_function(self, x):
         """
@@ -181,7 +215,7 @@ class Portfolio:
         ``float``
             The portfolio volatility on an annualized basis.
         """
-        return numpy.sqrt(numpy.multiply(x, self.sample_vol).dot(self.correlation_matrix).dot(numpy.transpose(numpy.multiply(x, self.sample_vol))))
+        return sqrt(multiply(x, self.sample_vol).dot(self.correl_matrix).dot(transpose(multiply(x, self.sample_vol))))
 
     def sharpe_ratio_function(self, x):
         """
@@ -197,7 +231,7 @@ class Portfolio:
         ``float``
             The portfolio sharpe ratio on an annualized basis.
         """
-        return (numpy.dot(x, self.mean_return) - self.risk_free_rate) / (self.volatility_function(x))
+        return (dot(x, self.mean_return) - self.risk_free_rate) / (self.volatility_function(x))
 
     def percentile_function(self, x, time, prob):
         """
@@ -205,18 +239,18 @@ class Portfolio:
 
         Parameters
         ----------
-        1. **x**: list
-            Vector representing the allocation of each asset in the portfolio. Must be preserve the order of `portfolio.tickers`, i.e. each element's index should map to each element of the `portfolio.tickers`'s list.
+        1. **x**: ``list``
+            Vector representing the allocation of each asset in the portfolio. Must be preserve the order of `self.tickers`, i.e. each element's index should map to each element of the `self.tickers`'s list.
         2: **time**: ``float``
             time horizon (in years) of the value at risk, i.e. the period of time into the future at which the value at risk is being calculated 
         3. **prob**: ``float``
             percentile desired
         """
         portfolio_return = self.return_function(x) * time
-        portfolio_volatility = self.volatility_function(x) * numpy.sqrt(time)
+        portfolio_volatility = self.volatility_function(x) * sqrt(time)
 
-        return probability.percentile(S0=1, vol=portfolio_volatility, ret=portfolio_return,
-                                      expiry=time, percentile=prob)
+        return percentile(S0=1, vol=portfolio_volatility, ret=portfolio_return,
+                          expiry=time, prob=prob)
 
     def conditional_value_at_risk_function(self, x, time, prob):
         """
@@ -226,16 +260,16 @@ class Portfolio:
         ----------
         1. **x**: ``list``
             an array of decimals representing percentage allocations of the portfolio. Must preserve order with `self.tickers`.
-        2.**time**: ``float``
+        2. **time**: ``float``
             time horizon (in years) of the value at risk, i.e. the period of time into the future at which the value at risk is being calculated.
         3. **prob**: ``float``
             desired probability of loss.
         """
         portfolio_return = self.return_function(x) * time
-        portfolio_volatility = self.volatility_function(x) * numpy.sqrt(time)
+        portfolio_volatility = self.volatility_function(x) * sqrt(time)
         value_at_risk = self.percentile_function(x=x, time=time, prob=prob)
-        return (1 - probability.conditional_expected_value(S0=1, vol=portfolio_volatility, ret=portfolio_return,
-                                                           expiry=time, conditional_value=value_at_risk))
+        return (1 - conditional_expected_value(S0=1, vol=portfolio_volatility, ret=portfolio_return,
+                                               expiry=time, conditional_value=value_at_risk))
 
     def get_init_guess(self):
         length = len(self.tickers)
@@ -255,35 +289,34 @@ class Portfolio:
 
     def get_target_return_constraint(self, x):
         if self.target_return is not None:
-            return (numpy.dot(x, self.mean_return) - self.target_return)
+            return (dot(x, self.mean_return) - self.target_return)
         return None
 
-    def calculate_approximate_shares(self, x, total, latest_prices=None):
+    @staticmethod
+    def calculate_approximate_shares(x, total, latest_prices: Dict[str, float]):
+        """
+
+        Parameters
+        ----------
+        1. **x**: 
+        2. **total**:
+        3. **latest_prices**: ``Dict[str, float]``. 
+            Dictionary of latest asset prices. Must preserve order with the allocation **x**, i.e. the *i*-th element of **x** must correspond to the same asset as the *i*-th element of the **latest_prices**.
+        """
         shares = []
         for i, item in enumerate(x):
-            if latest_prices is not None:
-                price = latest_prices[i]
-            elif self.sample_prices is not None:
-                price = self.sample_prices[self.tickers[i]][0]
-            else:
-                price = services.get_daily_price_latest(self.tickers[i])
-
+            price = list(latest_prices.values())[i]
             share = Decimal(item) * Decimal(total) / Decimal(price)
-            shares.append(math.trunc(share))
-
+            shares.append(trunc(share))
         return shares
 
-    def calculate_actual_total(self, x, total, latest_prices=None):
+    @staticmethod
+    def calculate_actual_total(x, total, latest_prices):
         actual_total = 0
-        shares = self.calculate_approximate_shares(
+        shares = Portfolio.calculate_approximate_shares(
             x=x, total=total, latest_prices=latest_prices)
         for i, item in enumerate(shares):
-            if latest_prices is not None:
-                price = latest_prices[i]
-            elif self.sample_prices is not None:
-                price = self.sample_prices[self.tickers[i]][0]
-            else:
-                price = services.get_daily_price_latest(self.tickers[i])
+            price = list(latest_prices.values())[i]
             portion = Decimal(item) * Decimal(price)
             actual_total = actual_total + portion
         return actual_total
