@@ -22,18 +22,50 @@ import io
 import json
 import csv
 import zipfile
-from typing import Any, Dict
+from typing import Any, Dict, Union
 import requests
 
-from scrilla import settings, errors
-from scrilla.static import functions, keys, constants
+from scrilla import settings
+from scrilla.static import keys, constants, formats
 from scrilla.util import outputter, helper
 
-logger = outputter.Logger("files", settings.LOG_LEVEL)
+logger = outputter.Logger("scrilla.files", settings.LOG_LEVEL)
 
 static_tickers_blob, static_econ_blob, static_crypto_blob = None, None, None
 
 
+def memory_json_skeleton() -> dict:
+    return {
+        'static': False,
+        'cache': {
+            'sqlite': {
+                'prices': False,
+                'interest': False,
+                'correlations': False,
+                'profile': False
+            },
+            'dynamodb': {
+                'prices': False,
+                'interest': False,
+                'correlations': False,
+                'profile': False
+            }
+        }
+    }
+
+
+def save_memory_json(persist: Union[dict, None] = None):
+    if persist is None:
+        persist = get_memory_json()
+    save_file(persist, settings.MEMORY_FILE)
+
+
+def get_memory_json():
+    if os.path.isfile(settings.MEMORY_FILE):
+        return load_file(settings.MEMORY_FILE)
+    return memory_json_skeleton()
+
+
 def load_file(file_name: str) -> Any:
     """
     Infers the file extensions from the provided `file_name` and parses the file appropriately. 
@@ -48,14 +80,18 @@ def load_file(file_name: str) -> Any:
 
 def save_file(file_to_save: Dict[str, Any], file_name: str) -> bool:
     ext = file_name.split('.')[-1]
-    with open(file_name, 'w') as outfile:
-        if ext == "json":
-            json.dump(file_to_save, outfile)
-        elif ext == "csv":
-            # TODO: assume input is dict since ll functions in library return dict.
-            writer = csv.DictWriter(outfile, file_to_save.keys())
-            writer.writeheader()
-        # TODO: implement other file saving extensions.
+    try:
+        with open(file_name, 'w') as outfile:
+            if ext == "json":
+                json.dump(file_to_save, outfile)
+            elif ext == "csv":
+                # TODO: assume input is dict since ll functions in library return dict.
+                writer = csv.DictWriter(outfile, file_to_save.keys())
+                writer.writeheader()
+            # TODO: implement other file saving extensions.
+        return True
+    except Exception as e:
+        return False
 
 
 def set_credentials(value: str, which_key: str) -> bool:
@@ -122,72 +158,70 @@ def parse_csv_response_column(column: int, url: str, firstRowHeader: str = None,
 
 def init_static_data():
     """
-    Initializes the three static files defined in  settings: `STATIC_TICKERS_FILE`, `STATIC_CRYPTO_FILE` and `STATIC_ECON_FILE`. The data for these files is retrieved from the service managers. While this function blurs the lines between file management and service management, the function has been included in the `files.py` module rather than the `services.py` module due the unique response types of static metadata. All metadata is returned as a csv or zipped csvs. These responses require specialized functions. Moreover, these files should only be initialized the first time the application executes. Subsequent executions will refer to their cached versions residing in the local filesytems. 
+    Initializes the three static files defined in  settings: `scrilla.settings.STATIC_TICKERS_FILE`, `scrilla.settings.STATIC_CRYPTO_FILE` and `scrilla.settings.STATIC_ECON_FILE`. The data for these files is retrieved from the service managers. While this function blurs the lines between file management and service management, the function has been included in the `files.py` module rather than the `services.py` module due the unique response types of static metadata. All metadata is returned as a csv or zipped csvs. These responses require specialized functions. Moreover, these files should only be initialized the first time the application executes. Subsequent executions will refer to their cached versions residing in the local filesytems. 
 
     Raises
     ------
-    1. **scrilla.errors.ConfigurationError**
-        If `scrilla.settings.PRICE_MANAGER` and `scrilla.settings.STAT_MANAGER` are not configured or are incorrectly configured through the environment variables `PRICE_MANAGER` and `STAT_MANAGER`, the function will throw this error.
+    1. `scrilla.errors.ConfigurationError` :
+        If `scrilla.settings.PRICE_MANAGER` and `scrilla.settings.STAT_MANAGER` are not configured or are incorrectly configured through the environment variables `scrilla.settings.PRICE_MANAGER` and `scrilla.settings.STAT_MANAGER`, the function will throw this error.
     """
-    global static_tickers_blob
-    global static_econ_blob
-    global static_crypto_blob
 
-    if ((not os.path.isfile(settings.STATIC_ECON_FILE)) or
-            (not os.path.isfile(settings.STATIC_TICKERS_FILE)) or
-            (not os.path.isfile(settings.STATIC_CRYPTO_FILE))):
+    memory = get_memory_json()
 
-        logger.info(
-            'Initializing static data. Please wait. This may take a moment...')
+    if not memory['static']:
+        global static_tickers_blob
+        global static_econ_blob
+        global static_crypto_blob
 
         # grab ticker symbols and store in STATIC_DIR
-        if not os.path.isfile(settings.STATIC_TICKERS_FILE):
-            if settings.PRICE_MANAGER == "alpha_vantage":
-                service_map = keys.keys["SERVICES"]["PRICES"]["ALPHA_VANTAGE"]["MAP"]
-                logger.debug(
-                    f'Missing {settings.STATIC_TICKERS_FILE}, querying \'{settings.PRICE_MANAGER}\'')
+        if (
+            settings.PRICE_MANAGER == "alpha_vantage" and
+            not os.path.isfile(settings.STATIC_TICKERS_FILE)
+        ):
+            service_map = keys.keys["SERVICES"]["PRICES"]["ALPHA_VANTAGE"]["MAP"]
+            logger.debug(
+                f'Missing {settings.STATIC_TICKERS_FILE}, querying \'{settings.PRICE_MANAGER}\'', 'init_static_data')
 
-                # TODO: services calls should be in services.py! need to put this and the helper method
-                #       into services.py in the future.
-                query = f'{service_map["PARAMS"]["FUNCTION"]}={service_map["ARGUMENTS"]["EQUITY_LISTING"]}'
-                url = f'{settings.AV_URL}?{query}&{service_map["PARAMS"]["KEY"]}={settings.av_key()}'
-                static_tickers_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_TICKERS_FILE,
-                                                                firstRowHeader=service_map['KEYS']['EQUITY']['HEADER'])
-
-            raise errors.ConfigurationError(
-                "No PRICE_MANAGER set in .env file!")
+            # TODO: services calls should be in services.py! need to put this and the helper method
+            #       into services.py in the future.
+            query = f'{service_map["PARAMS"]["FUNCTION"]}={service_map["ARGUMENTS"]["EQUITY_LISTING"]}'
+            url = f'{settings.AV_URL}?{query}&{service_map["PARAMS"]["KEY"]}={settings.av_key()}'
+            static_tickers_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_TICKERS_FILE,
+                                                            firstRowHeader=service_map['KEYS']['EQUITY']['HEADER'])
 
         # grab crypto symbols and store in STATIC_DIR
-        if not os.path.isfile(settings.STATIC_CRYPTO_FILE):
-            if settings.PRICE_MANAGER == "alpha_vantage":
-                service_map = keys.keys["SERVICES"]["PRICES"]["ALPHA_VANTAGE"]["MAP"]
-                logger.debug(
-                    f'Missing {settings.STATIC_CRYPTO_FILE}, querying \'{settings.PRICE_MANAGER}\'.')
-                url = settings.AV_CRYPTO_LIST
-                static_crypto_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_CRYPTO_FILE,
-                                                               firstRowHeader=service_map['KEYS']['CRYPTO']['HEADER'])
-            raise errors.ConfigurationError(
-                "No PRICE_MANAGER set in .env file!")
+        if (
+            settings.PRICE_MANAGER == "alpha_vantage" and
+            not os.path.isfile(settings.STATIC_CRYPTO_FILE)
+        ):
+            service_map = keys.keys["SERVICES"]["PRICES"]["ALPHA_VANTAGE"]["MAP"]
+            logger.debug(
+                f'Missing {settings.STATIC_CRYPTO_FILE}, querying \'{settings.PRICE_MANAGER}\'.', 'init_static_data')
+            url = settings.AV_CRYPTO_LIST
+            static_crypto_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_CRYPTO_FILE,
+                                                           firstRowHeader=service_map['KEYS']['CRYPTO']['HEADER'])
 
         # grab econominc indicator symbols and store in STATIC_DIR
-        if not os.path.isfile(settings.STATIC_ECON_FILE):
-            if settings.STAT_MANAGER == "quandl":
-                service_map = keys.keys["SERVICES"]["STATISTICS"]["QUANDL"]["MAP"]
+        if (
+            settings.STAT_MANAGER == "quandl" and
+            not os.path.isfile(settings.STATIC_ECON_FILE)
+        ):
+            service_map = keys.keys["SERVICES"]["STATISTICS"]["QUANDL"]["MAP"]
 
-                logger.debug(
-                    f'Missing {settings.STATIC_ECON_FILE}, querying \'{settings.STAT_MANAGER}\'.')
+            logger.debug(
+                f'Missing {settings.STATIC_ECON_FILE}, querying \'{settings.STAT_MANAGER}\'.', 'init_static_data')
 
-                query = f'{service_map["PATHS"]["FRED"]}/{service_map["PARAMS"]["METADATA"]}'
-                url = f'{settings.Q_META_URL}/{query}?{service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
-                static_econ_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_ECON_FILE,
-                                                             firstRowHeader=service_map["KEYS"]["HEADER"],
-                                                             zipped=service_map["KEYS"]["ZIPFILE"])
+            query = f'{service_map["PATHS"]["FRED"]}/{service_map["PARAMS"]["METADATA"]}'
+            url = f'{settings.Q_META_URL}/{query}?{service_map["PARAMS"]["KEY"]}={settings.Q_KEY}'
+            static_econ_blob = parse_csv_response_column(column=0, url=url, savefile=settings.STATIC_ECON_FILE,
+                                                         firstRowHeader=service_map["KEYS"]["HEADER"],
+                                                         zipped=service_map["KEYS"]["ZIPFILE"])
 
-            raise errors.ConfigurationError(
-                "No STAT_MANAGER set in .env file!")
+        memory['static'] = True
+        save_memory_json(memory)
 
     else:
-        logger.debug('Static data already initialized!')
+        logger.debug('Static data already initialized!', 'init_static_data')
 
 
 def get_static_data(static_type):
@@ -226,16 +260,17 @@ def get_static_data(static_type):
         return None
 
     if blob is not None:
-        logger.debug(f'Found in-memory {static_type} symbols.')
+        logger.verbose(
+            f'Found in-memory {static_type} symbols.', 'get_static_data')
         return blob
 
     if path is not None:
         if not os.path.isfile(path):
             init_static_data()
-        logger.debug(f'Loading in cached {static_type} symbols.')
+        logger.verbose(
+            f'Loading in cached {static_type} symbols.', 'get_static_data')
 
         ext = path.split('.')[-1]
-        print(ext)
 
         with open(path, 'r') as infile:
             if ext == "json":
@@ -313,19 +348,20 @@ def get_watchlist() -> list:
     -----------
     Retrieves the list of watchlisted equity ticker symbols saved in /data/common/watchlist.json.
     """
-    logger.debug('Loading in Watchlist symbols.')
+    logger.debug('Loading in Watchlist symbols.', 'get_watchlist')
 
     if os.path.isfile(settings.COMMON_WATCHLIST_FILE):
-        logger.debug('Watchlist found.')
+        logger.debug('Watchlist found.', 'get_watchlist')
         ext = settings.COMMON_WATCHLIST_FILE.split('.')[-1]
         with open(settings.COMMON_WATCHLIST_FILE, 'r') as infile:
             if ext == "json":
                 watchlist = json.load(infile)
-                logger.debug('Watchlist loaded in JSON format.')
+                logger.verbose(
+                    'Watchlist loaded in JSON format.', 'get_watchlist')
 
             # TODO: implement other file loading exts
     else:
-        logger.info('Watchlist not found.')
+        logger.error('Watchlist not found.', 'get_watchlist')
         watchlist = []
 
     return watchlist
@@ -337,14 +373,15 @@ def add_watchlist(new_tickers: list) -> None:
     -----------
     Retrieves the list of watchlisted equity ticker symbols saved in /data/common/watchlist.json and then appends to it the list of tickers supplied as arguments. After appending, the list is sorted in alphabetical order. The tickers to add must exist in the /data/static/tickers.json file in order to be added to the watchlist, i.e. the tickers must have price histories that can be retrieved (the static file tickers.json contains a list of all equities with retrievable price histories.) \n \n 
     """
-    logger.debug('Saving tickers to Watchlist')
+    logger.debug('Saving tickers to Watchlist', 'add_watchlist')
 
     current_tickers = get_watchlist()
     all_tickers = get_static_data(keys.keys['ASSETS']['EQUITY'])
 
     for ticker in new_tickers:
         if ticker not in current_tickers and ticker in all_tickers:
-            logger.debug(f'New ticker being added to Watchlist: {ticker}')
+            logger.debug(
+                f'New ticker being added to Watchlist: {ticker}', 'add_watchlist')
             current_tickers.append(ticker)
 
     current_tickers = sorted(current_tickers)
@@ -356,37 +393,29 @@ def add_watchlist(new_tickers: list) -> None:
         # TODO: implement other file extensions
 
 
-def save_profiles(profiles: dict, file_name: str):
-    save_format = functions.format_profiles(profiles)
+def save_allocation(allocation, portfolio, file_name, investment=None, latest_prices=None):
+    save_format = formats.format_allocation(
+        allocation=allocation, portfolio=portfolio, investment=investment, latest_prices=latest_prices)
     save_file(file_to_save=save_format, file_name=file_name)
 
 
-def save_allocation(allocation, portfolio, file_name, investment=None):
-    save_format = functions.format_allocation(
-        allocation=allocation, portfolio=portfolio, investment=investment)
-    save_file(file_to_save=save_format, file_name=file_name)
-
-
-def save_frontier(portfolio, frontier, file_name, investment=None):
-    save_format = functions.format_frontier(
-        portfolio=portfolio, frontier=frontier, investment=investment)
+def save_frontier(portfolio, frontier, file_name, investment=None, latest_prices=None):
+    save_format = formats.format_frontier(
+        portfolio=portfolio, frontier=frontier, investment=investment, latest_prices=latest_prices)
     save_file(file_to_save=save_format, file_name=file_name)
 
 
 def save_moving_averages(tickers, averages_output, file_name):
-    save_format = functions.format_moving_averages(
+    save_format = formats.format_moving_averages(
         tickers=tickers, averages_output=averages_output)
     save_file(file_to_save=save_format, file_name=file_name)
 
 
 def save_correlation_matrix(tickers, correlation_matrix, file_name):
-    save_format = functions.format_correlation_matrix(
+    save_format = formats.format_correlation_matrix(
         tickers=tickers, correlation_matrix=correlation_matrix)
     save_file(file_to_save=save_format, file_name=file_name)
 
-################################################
-# FILE MANAGEMENT FUNCTIONS
-
 
 def clear_directory(directory, retain=True):
     """
