@@ -12,17 +12,6 @@
 # You should have received a copy of the GNU General Public License
 # along with scrilla.  If not, see <https://www.gnu.org/licenses/>
 # or <https://github.com/chinchalinchin/scrilla/blob/develop/main/LICENSE>.
-
-import datetime
-from typing import Callable, Dict, List, Union
-from PySide6 import QtCore, QtWidgets, QtGui
-
-from scrilla import settings
-from scrilla.util import helper
-from scrilla.gui import utilities
-from scrilla.static import definitions
-from scrilla.gui.widgets import factories
-
 """
 A series of classes that all inherit from ``PySide6.QtWidgets.QWidget` and build up in sequence the necessary functionality to grab and validate user input, calculate and display results, etc. These widgets are not directly displayed on the GUI; rather, they are used as building blocks by the `scrilla.gui.functions` module, to create widgets specifically for application functions.
 
@@ -38,6 +27,15 @@ The idea behind the module is to hide as much widget initialization and styling
 
 Widgets have styles applied to them through the `scrilla.styles.app.qss` stylesheet. Widgets are layered over one another in a hierarchy described by: `root` -> `child` -> `grand-child` -> `great-grand-child` -> etc. Each layers has a theme that descends down the material color scheme hex codes in sequence with its place in the layer hierarchy. See `scrilla.gui.formats` for more information. 
 """
+import datetime
+from typing import Callable, Dict, List, Union
+from PySide6 import QtCore, QtWidgets, QtGui
+
+from scrilla import settings
+from scrilla.util import helper
+from scrilla.gui import utilities
+from scrilla.static import definitions
+from scrilla.gui.widgets import factories
 
 SYMBOLS_LIST = "list"
 """Constant passed into `scrilla.gui.widgets.components.ArgumentWidget` to initialize an input control allowing user to input list of ticker symbols"""
@@ -179,14 +177,14 @@ class ArgumentWidget(QtWidgets.QWidget):
                 group_key = self.group_definitions[definition]['group']
                 group_name = definitions.GROUP_DICT[group_key]
 
-                if group_name not in groups.keys():
+                if group_name not in groups:
                     groups[group_name] = [definition]
                 else:
                     groups[group_name].append(definition)
 
-            for group in groups:
-                self.group_control_widgets[group] = factories.group_widget_factory(
-                    groups[group], group)
+            for group_name, group in groups.items():
+                self.group_control_widgets[group_name] = factories.group_widget_factory(
+                    group, group_name)
 
     def _arrange_widgets(self):
         """
@@ -260,7 +258,7 @@ class ArgumentWidget(QtWidgets.QWidget):
             return None
 
         if type(widget) is QtWidgets.QDateEdit:
-            return widget.date().toPython().date()
+            return widget.date().toPython()
         if type(widget) is QtWidgets.QLineEdit:
             if type(widget.validator()) is QtGui.QIntValidator:
                 return int(widget.text())
@@ -376,11 +374,11 @@ class TableWidget(QtWidgets.QWidget):
         self.table.setColumnCount(len(columns))
         self.table.setHorizontalHeaderLabels(columns)
         self.table.setVerticalHeaderLabels(rows)
-        self.download_button.show()
 
     def show_table(self):
         self.table.resizeColumnsToContents()
         self.table.show()
+        self.download_button.show()
 
     @QtCore.Slot()
     def show_file_dialog(self) -> None:
@@ -390,9 +388,12 @@ class TableWidget(QtWidgets.QWidget):
         filename = None
         if file_path.exec_() == QtWidgets.QDialog.Accepted:
             filename = file_path.selectedFiles()
-        if filename is not None and len(filename) > 0:
-            if settings.FILE_EXT == 'json':
-                utilities.download_table_to_json(self.table, filename[0])
+        if (
+            filename is not None
+            and len(filename) > 0
+            and settings.FILE_EXT == 'json'
+        ):
+            utilities.download_table_to_json(self.table, filename[0])
 
 
 class GraphWidget(QtWidgets.QWidget):
