@@ -18,14 +18,14 @@ from PySide6 import QtGui, QtCore, QtWidgets
 
 
 from scrilla import settings, services
-from scrilla.static import keys, definitions
+from scrilla.static import keys
 # TODO: conditional import based on ANALYSIS_MODE
-from scrilla.analysis import estimators, markets, optimizer
+from scrilla.analysis import estimators, markets, optimizer, plotter
 from scrilla.analysis.models.geometric import statistics
 from scrilla.analysis.objects.portfolio import Portfolio
 from scrilla.analysis.objects.cashflow import Cashflow
 
-from scrilla.util import dater, outputter, helper, plotter
+from scrilla.util import dater, outputter, helper
 
 from scrilla.gui import formats, utilities
 from scrilla.gui.widgets import factories, components
@@ -146,7 +146,7 @@ class YieldCurveWidget(components.SkeletonWidget):
         yield_curve = {}
         start_date = dater.this_date_or_last_trading_date(
             self.arg_widget.get_control_input('start_date'))
-        start_string = dater.date_to_string(start_date)
+        start_string = dater.to_string(start_date)
         yield_curve[start_string] = []
         for maturity in keys.keys['YIELD_CURVE']:
             rate = services.get_daily_interest_history(maturity=maturity,
@@ -269,17 +269,25 @@ class RiskProfileWidget(components.SkeletonWidget):
                                                       columns=['Return', 'Volatility', 'Sharpe', 'Beta', 'Equity Cost'])
 
         profiles = {}
+        start_date = start_date = self.arg_widget.get_control_input(
+            'start_date')
+        end_date = self.arg_widget.get_control_input('end_date')
         for i, symbol in enumerate(symbols):
             profiles[symbol] = statistics.calculate_risk_return(ticker=symbol,
-                                                                start_date=self.arg_widget.get_control_input(
-                                                                    'start_date'),
-                                                                end_date=self.arg_widget.get_control_input('end_date'))
+                                                                start_date=start_date,
+                                                                end_date=end_date)
             profiles[symbol][keys.keys['APP']['PROFILE']
-                             ['SHARPE']] = markets.sharpe_ratio(symbol)
+                             ['SHARPE']] = markets.sharpe_ratio(ticker=symbol,
+                                                                start_date=start_date,
+                                                                end_date=end_date)
             profiles[symbol][keys.keys['APP']['PROFILE']
-                             ['BETA']] = markets.market_beta(symbol)
+                             ['BETA']] = markets.market_beta(ticker=symbol,
+                                                             start_date=start_date,
+                                                             end_date=end_date)
             profiles[symbol][keys.keys['APP']['PROFILE']
-                             ['EQUITY']] = markets.cost_of_equity(symbol)
+                             ['EQUITY']] = markets.cost_of_equity(ticker=symbol,
+                                                                  start_date=start_date,
+                                                                  end_date=end_date)
 
             formatted_profile = formats.format_profile(profiles[symbol])
 
@@ -367,6 +375,7 @@ class CorrelationWidget(components.SkeletonWidget):
         self.arg_widget.prime()
         self.table_widget.table.clear()
         self.table_widget.table.hide()
+        self.table_widget.download_button.hide()
 
 
 class OptimizerWidget(components.SkeletonWidget):
@@ -415,6 +424,8 @@ class OptimizerWidget(components.SkeletonWidget):
             self.title.setText(formats.format_allocation_profile_title(
                 allocation, this_portfolio))
 
+            prices = services.get_daily_prices_latest(tickers=symbols)
+
             if investment is None:
                 self.table_widget.init_table(
                     rows=symbols, columns=['Allocation'])
@@ -422,7 +433,7 @@ class OptimizerWidget(components.SkeletonWidget):
                 self.table_widget.init_table(rows=symbols, columns=[
                                              'Allocation', 'Shares'])
                 shares = this_portfolio.calculate_approximate_shares(
-                    allocation, float(investment))
+                    allocation, float(investment), prices)
 
             for i in range(len(symbols)):
                 item = factories.atomic_widget_factory(
@@ -441,7 +452,7 @@ class OptimizerWidget(components.SkeletonWidget):
             self.arg_widget.fire()
 
         else:
-            print('something went wrong')
+            print('error handling goes here')
 
     @QtCore.Slot()
     def clear(self):
@@ -519,7 +530,9 @@ class MovingAverageWidget(components.SkeletonWidget):
         self.arg_widget = components.ArgumentWidget(calculate_function=self.calculate,
                                                     clear_function=self.clear,
                                                     controls=self.controls,
-                                                    layer=utilities.get_next_layer(self.objectName()))
+                                                    layer=utilities.get_next_layer(
+                                                        self.objectName()),
+                                                    mode=components.SYMBOLS_SINGLE)
         self.setLayout(QtWidgets.QHBoxLayout())
 
     def _arrange_widgets(self):
@@ -539,17 +552,13 @@ class MovingAverageWidget(components.SkeletonWidget):
         if self.graph_widget.figure.isVisible():
             self.graph_widget.figure.hide()
 
-        symbols = self.arg_widget.get_symbol_input()
-
-        moving_averages = statistics.calculate_moving_averages(tickers=self.arg_widget.get_symbol_input(),
+        moving_averages = statistics.calculate_moving_averages(ticker=self.arg_widget.get_symbol_input()[0],
                                                                start_date=self.arg_widget.get_control_input(
                                                                    'start_date'),
                                                                end_date=self.arg_widget.get_control_input('end_date'))
-        periods = [settings.MA_1_PERIOD,
-                   settings.MA_2_PERIOD, settings.MA_3_PERIOD]
-        plotter.plot_moving_averages(symbols=symbols,
-                                     averages_output=moving_averages,
-                                     periods=periods,
+
+        plotter.plot_moving_averages(ticker=self.arg_widget.get_symbol_input()[0],
+                                     averages=moving_averages,
                                      show=False,
                                      savefile=f'{settings.TEMP_DIR}/{keys.keys["GUI"]["TEMP"]["AVERAGES"]}')
         self.graph_widget.set_pixmap()
@@ -558,4 +567,4 @@ class MovingAverageWidget(components.SkeletonWidget):
     @QtCore.Slot()
     def clear(self):
         self.arg_widget.prime()
-        self.graph_widget.figure.hide()
+        self.graph_widget.clear()
