@@ -15,24 +15,35 @@
 
 """
 This module interfaces with the external services the program uses to hydrate with financial data. In the case of price and interest history, the functions in this module defer to the cache before making expensive HTTP requests. Statistical data retrieved from FRED and dividment payment histories are not persisted in the cache since most of the data is reported on an irregular basis and it is impossible to tell based on the date alone whether or not the cache is out of date.
+
+This module can be imported and used directly in other Python scripts if the API keys for the services have been set.
+
+```
+import os
+from scrilla.services import get_daily_price_history
+
+os.environ.setdefault('ALPHA_VANTAGE_KEY')
+prices = get_daily_price_history('AAPL')
+```
 """
 import itertools
 import time
 import requests
-from typing import Union
+from typing import Dict, List, Union
+import defusedxml.ElementTree as ET
 
 from datetime import date
 
-from scrilla import settings, errors, cache
+from scrilla import settings, cache
 from scrilla.static import keys, constants
-from scrilla.util import outputter, helper, dater
+from scrilla.util import errors, outputter, helper, dater
 
-logger = outputter.Logger("services", settings.LOG_LEVEL)
+logger = outputter.Logger("scrilla.services", settings.LOG_LEVEL)
 
 
 class StatManager():
     """
-    StatManager is an interface between the application and the external services that hydrate it with financial statistics data. This class gets instantiated on the level of the `scrilla.services` module with the value defined in `scrilla.settings.STAT_MANAGER`. This value is in turn defined by the value of the `STAT_MANAGER` environment variable. This value determines how the url is constructed, which API credentials get appended to the external query and the keys used to parse the response JSON container the statistical data.
+    StatManager is an interface between the application and the external services that hydrate it with financial statistics data. This class gets instantiated on the level of the `scrilla.services` module with the value defined in `scrilla.settings.STAT_MANAGER`. This value is in turn defined by the value of the `STAT_MANAGER` environment variable. This value determines how the url is constructed, which API credentials get appended to the external query and the keys used to parse the response JSON containing the statistical data.
 
     Attributes
     ----------
@@ -57,9 +68,13 @@ class StatManager():
             self.service_map = keys.keys["SERVICES"]["STATISTICS"]["QUANDL"]["MAP"]
             self.key = settings.Q_KEY
             self.url = settings.Q_URL
+        elif self._is_treasury():
+            self.service_map = keys.keys["SERVICES"]["STATISTICS"]["TREASURY"]["MAP"]
+            self.url = settings.TR_URL
+            self.key = None
         if self.service_map is None:
             raise errors.ConfigurationError(
-                'No STAT_MANAGER found in the parsed environment settings')
+                'No STAT_MANAGER found in the environment settings')
 
     def _is_quandl(self):
         """
@@ -76,6 +91,21 @@ class StatManager():
             return True
         return False
 
+    def _is_treasury(self):
+        """
+        Returns
+        -------
+        `bool`
+            `True` if this instace of `StatManager` is a US Treasury interface. `False` otherwise.
+
+        .. notes::
+            * This is for use within the class and probably won't need to be accessed outside of it. `StatManager` is intended to hide the data implementation from the rest of the library, i.e. it is ultimately agnostic about where the data comes where. It should never need to know `StatManger` is a Quandl interface. Just in case the library ever needs to populate its data from another source.
+
+        """
+        if self.genre == keys.keys['SERVICES']['STATISTICS']['TREASURY']['MANAGER']:
+            return True
+        return False
+
     def _construct_query(self, start_date: date, end_date: date) -> str:
         """
         Constructs and formats the query parameters for the external statistics service. Note, this method appends the API key to the query. Be careful with the returned value.
@@ -94,19 +124,26 @@ class StatManager():
 
         """
         query = ""
+
         if end_date is not None:
-            end_string = dater.date_to_string(end_date)
+            if self._is_treasury():
+                end_string = "all"
+            else:
+                end_string = dater.to_string(end_date)
             query += f'&{self.service_map["PARAMS"]["END"]}={end_string}'
 
-        if start_date is not None:
-            start_string = dater.date_to_string(start_date)
+        if start_date is not None and not self._is_treasury():
+            start_string = dater.to_string(start_date)
             query += f'&{self.service_map["PARAMS"]["START"]}={start_string}'
 
-        if query:
-            logger.debug(f'StatManager Query (w/o key) = {query}')
-            return f'{query}&{self.service_map["PARAMS"]["KEY"]}={self.key}'
+        logger.debug(
+            f'StatManager Query (w/o key) = {query}', 'StatManager._construct_query')
 
-        return f'{self.service_map["PARAMS"]["KEY"]}={self.key}'
+        if self.service_map["PARAMS"].get("KEY", None) is not None:
+            if query:
+                return f'{query}&{self.service_map["PARAMS"]["KEY"]}={self.key}'
+            return f'{self.service_map["PARAMS"]["KEY"]}={self.key}'
+        return query
 
     def _construct_stat_url(self, symbol: str, start_date: date, end_date: date):
         """
@@ -151,6 +188,8 @@ class StatManager():
             * The URL returned by this method will always contain a query for a historical range of US Treasury Yields, i.e. this method is specifically for queries involving the "Risk-Free" (right? right? *crickets*) Yield Curve. 
         """
         url = f'{self.url}/{self.service_map["PATHS"]["YIELD"]}?'
+        if self._is_treasury():
+            url += f'{self.service_map["PARAMS"]["DATA"]}={self.service_map["ARGUMENTS"]["DAILY"]}'
         url += self._construct_query(start_date=start_date, end_date=end_date)
         return url
 
@@ -169,13 +208,65 @@ class StatManager():
     def get_interest_rates(self, start_date, end_date):
         url = self._construct_interest_url(
             start_date=start_date, end_date=end_date)
-        response = requests.get(url).json()
-
-        raw_interest = response[self.service_map["KEYS"]
-                                ["FIRST_LAYER"]][self.service_map["KEYS"]["SECOND_LAYER"]]
         formatted_interest = {}
-        for rate in raw_interest:
-            formatted_interest[rate[0]] = rate[1:]
+
+        if self._is_quandl():
+            response = requests.get(url)
+
+            response = response.json()
+            raw_interest = response[self.service_map["KEYS"]
+                                    ["FIRST_LAYER"]][self.service_map["KEYS"]["SECOND_LAYER"]]
+            for rate in raw_interest:
+                formatted_interest[rate[0]] = rate[1:]
+
+        elif self._is_treasury():
+            # this is ugly, but it's the government's fault for not supporting an API
+            # from this century.
+
+            def __paginate(page_no, page_url):
+                page_url = f'{page_url}&{self.service_map["PARAMS"]["PAGE"]}={page_no}'
+                logger.verbose(
+                    f'Paginating: {page_url}', 'StatManager.get_interest_rates.__paginate')
+                page_response = ET.fromstring(requests.get(page_url).text)
+                return page_no - 1, page_response
+
+            record_time = dater.business_days_between(
+                constants.constants['YIELD_START_DATE'], end_date, True)
+            # subtract to reindex to 0
+            pages = record_time // self.service_map["KEYS"]["PAGE_LENGTH"] - 1
+            pages += 1 if record_time % self.service_map["KEYS"]["PAGE_LENGTH"] > 0 else 0
+            page = pages
+
+            logger.debug(
+                f'Sorting through {pages} pages of Treasury data', 'StatManager.get_interest_rates')
+            logger.debug(
+                f'Days from {dater.to_string(end_date)} to start of Treasury record: {record_time}', 'StatManager.get_interest_rates')
+
+            while True:
+                page, response = __paginate(page, url)
+                first_layer = response.findall(
+                    self.service_map["KEYS"]["FIRST_LAYER"])
+
+                if page >= 0:
+                    for child in first_layer:
+                        xpath = f'{self.service_map["KEYS"]["RATE_XPATH"]}{self.service_map["KEYS"]["DATE"]}'
+                        this_date = dater.parse(child.find(xpath).text)
+
+                        if start_date <= this_date <= end_date:
+                            date_string = dater.to_string(this_date)
+                            formatted_interest[date_string] = []
+
+                            for maturity in self.service_map["YIELD_CURVE"].values():
+                                interest = child.find(
+                                    f'{self.service_map["KEYS"]["RATE_XPATH"]}{maturity}').text
+                                formatted_interest[date_string].append(
+                                    float(interest))
+
+                        if len(formatted_interest) >= dater.business_days_between(start_date, end_date, True):
+                            return formatted_interest
+                else:
+                    break
+
         return formatted_interest
 
     @staticmethod
@@ -216,7 +307,8 @@ class DividendManager():
     def _construct_url(self, ticker):
         query = f'{ticker}/{self.service_map["PATHS"]["DIV"]}/{self.service_map["PARAMS"]["FULL"]}'
         url = f'{self.url}/{query}?{self.service_map["PARAMS"]["KEY"]}={self.key}'
-        logger.debug(f'DivManager Query (w/o key) = {query}')
+        logger.debug(
+            f'DivManager Query (w/o key) = {query}', 'DividendManager._construct_url')
         return url
 
     def get_dividends(self, ticker):
@@ -225,9 +317,9 @@ class DividendManager():
         formatted_response = {}
 
         for item in response:
-            date = str(item[self.service_map['KEYS']['DATE']])
+            this_date = str(item[self.service_map['KEYS']['DATE']])
             div = item[self.service_map['KEYS']['AMOUNT']]
-            formatted_response[date] = div
+            formatted_response[this_date] = div
 
         return formatted_response
 
@@ -293,10 +385,11 @@ class PriceManager():
 
         auth_query = query + f'&{self.service_map["PARAMS"]["KEY"]}={self.key}'
         url = f'{self.url}?{auth_query}'
-        logger.debug(f'PriceManager query (w/o key) = {query}')
+        logger.debug(
+            f'PriceManager query (w/o key) = {query}', 'PriceManager._construct_url')
         return url
 
-    def get_prices(self, ticker, start_date, end_date, asset_type):
+    def get_prices(self, ticker: str, start_date: date, end_date: date, asset_type: str):
         """
         Retrieve prices from external service.
 
@@ -304,14 +397,28 @@ class PriceManager():
         ----------
         1. **ticker** : ``str``
             Ticker symbol of the asset whose prices are being retrieved.
-        2. **asset_type** : ``str``
+        2. **start_date**; ``str``
+        3. **end_date**: ``str``
+        4. **asset_type** : ``str``
             Asset type of the asset whose prices are being retrieved. Options are statically
             accessible in the `scrillla.static` module dictionary `scrilla.keys.keys['ASSETS']`.
 
         Returns
         -------
-        ``dict``: `{ 'date': value, 'date': value, ...}`
-            Dictionary of prices with date as key, ordered from latest to earliest.
+        ``Dict[str, Dict[str, float]]``
+        ```
+            prices = { 
+                        'date': { 
+                            'open': value, 
+                            'close': value
+                        },  
+                        'date': { 
+                            'open': value,
+                            'close': value
+                        }
+            }
+        ```
+         Dictionary of prices with date as key, ordered from latest to earliest.
 
         Raises
         ------
@@ -338,11 +445,11 @@ class PriceManager():
 
         while first_element == self.service_map['ERRORS']['RATE_THROTTLE']:
             if first_pass:
-                logger.comment(
-                    f'{self.genre} API rate limit per minute exceeded. Waiting...')
+                logger.info(
+                    f'{self.genre} API rate limit per minute exceeded. Waiting...', 'PriceManager.get_prices')
                 first_pass = False
             else:
-                logger.comment('Waiting...')
+                logger.info('Waiting...', 'PriceManager.get_prices')
 
             time.sleep(constants.constants['BACKOFF_PERIOD'])
             response = requests.get(url).json()
@@ -356,9 +463,9 @@ class PriceManager():
             start_date=start_date, end_date=end_date, asset_type=asset_type, prices=response)
         format_prices = {}
         for this_date in prices:
-            close_price = self._parse_price_from_date(prices=prices, date=this_date, asset_type=asset_type,
+            close_price = self._parse_price_from_date(prices=prices, this_date=this_date, asset_type=asset_type,
                                                       which_price=keys.keys['PRICES']['CLOSE'])
-            open_price = self._parse_price_from_date(prices=prices, date=this_date, asset_type=asset_type,
+            open_price = self._parse_price_from_date(prices=prices, this_date=this_date, asset_type=asset_type,
                                                      which_price=keys.keys['PRICES']['OPEN'])
             format_prices[this_date] = {
                 keys.keys['PRICES']['OPEN']: open_price, keys.keys['PRICES']['CLOSE']: close_price}
@@ -392,14 +499,14 @@ class PriceManager():
             If one of the settings is improperly configured or one of the environment variables was unable to be parsed from the environment, this error will be thrown.
         """
 
-        # NOTE: only really needed for `alpha_vantage` responses so far, due to the fact AlphaVantage either returns everything or 100 days or prices.
-        # shouldn't need to verify genre anyway, since using service_map and service_map should abstract the response away.
+        # NOTE: only really needed for `alpha_vantage` responses so far, due to the fact AlphaVantage either returns everything or 100 days of prices.
+        # shouldn't need to verify genre anyway, since using service_map should abstract the response away (hopefully).
         if self.genre == keys.keys['SERVICES']['PRICES']['ALPHA_VANTAGE']['MANAGER']:
 
             # NOTE: Remember AlphaVantage is ordered current to earliest. END_INDEX is
             # actually the beginning of slice and START_INDEX is actually end of slice.
-            start_string, end_string = dater.date_to_string(
-                start_date), dater.date_to_string(end_date)
+            start_string, end_string = dater.to_string(
+                start_date), dater.to_string(end_date)
             if asset_type == keys.keys['ASSETS']['EQUITY']:
                 response_map = self.service_map['KEYS']['EQUITY']['FIRST_LAYER']
             elif asset_type == keys.keys['ASSETS']['CRYPTO']:
@@ -414,14 +521,14 @@ class PriceManager():
         raise errors.ConfigurationError(
             'No PRICE_MANAGER found in the parsed environment settings')
 
-    def _parse_price_from_date(self, prices: dict, date: date, asset_type: str, which_price: str) -> str:
+    def _parse_price_from_date(self, prices: Dict[str, Dict[str, float]], this_date: date, asset_type: str, which_price: str) -> str:
         """
         Parameters
         ----------
-        1. **prices** : ``dict``
+        1. **prices** : ``Dict[str, Dict[str, float]]``
             List containing the AlphaVantage response with the first layer peeled off, i.e.
             no metadata, just the date and prices.
-        2. **date**: ``str``
+        2. **date**: `date``
             String of the date to be parsed. Note: this is not a datetime.date object. String
             must be formatted `YYYY-MM-DD`
         3. **asset_type**: ``str``
@@ -444,29 +551,21 @@ class PriceManager():
         """
         if asset_type == keys.keys['ASSETS']['EQUITY']:
             if which_price == keys.keys['PRICES']['CLOSE']:
-                return prices[date][self.service_map['KEYS']['EQUITY']['CLOSE']]
+                return prices[this_date][self.service_map['KEYS']['EQUITY']['CLOSE']]
             if which_price == keys.keys['PRICES']['OPEN']:
-                return prices[date][self.service_map['KEYS']['EQUITY']['CLOSE']]
+                return prices[this_date][self.service_map['KEYS']['EQUITY']['OPEN']]
 
         elif asset_type == keys.keys['ASSETS']['CRYPTO']:
             if which_price == keys.keys['PRICES']['CLOSE']:
-                return prices[date][self.service_map['KEYS']['CRYPTO']['CLOSE']]
+                return prices[this_date][self.service_map['KEYS']['CRYPTO']['CLOSE']]
             if which_price == keys.keys['PRICES']['OPEN']:
-                return prices[date][self.service_map['KEYS']['CRYPTO']['OPEN']]
+                return prices[this_date][self.service_map['KEYS']['CRYPTO']['OPEN']]
 
         raise errors.InputValidationError(
             f'Verify {asset_type}, {which_price} are allowable values')
 
 
-price_manager = PriceManager(settings.PRICE_MANAGER)
-stat_manager = StatManager(settings.STAT_MANAGER)
-div_manager = DividendManager(settings.DIV_MANAGER)
-price_cache = cache.PriceCache()
-interest_cache = cache.InterestCache()
-
-
-def get_daily_price_history(ticker: str, start_date: Union[None, date] = None,
-                            end_date: Union[None, date] = None, asset_type: Union[None, str] = None) -> list:
+def get_daily_price_history(ticker: str, start_date: Union[None, date] = None, end_date: Union[None, date] = None, asset_type: Union[None, str] = None) -> Dict[str, Dict[str, float]]:
     """
     Wrapper around external service request for price data. Relies on an instance of `PriceManager` configured by `settings.PRICE_MANAGER` value, which in turn is configured by the `PRICE_MANAGER` environment variable, to hydrate with data. 
 
@@ -485,8 +584,27 @@ def get_daily_price_history(ticker: str, start_date: Union[None, date] = None,
 
     Returns
     ------
-    ``dict``: `{ 'date' : { 'open': value, 'close': value  }, 'date': { 'open' : value, 'close' : value }, ... }`
-        Dictionary with date strings formatted `YYYY-MM-DD` as keys and a nested dictionary containing the 'open' and 'close' price as values. Ordered from latest to earliest.
+    ``Dict[str, Dict[str, float]]`` : Dictionary with date strings formatted `YYYY-MM-DD` as keys and a nested dictionary containing the 'open' and 'close' price as values. Ordered from latest to earliest, e.g., 
+    ```
+        { 
+            'date' : 
+                { 
+                    'open': value, 
+                    'close': value  
+                }, 
+            'date': 
+                { 
+                    'open' : value, 
+                    'close' : value 
+                }, 
+            ... 
+        }
+    ```
+
+    Raises
+    ------
+    1. **scrilla.errors.PriceError**
+        If no sample prices can be retrieved, this error is thrown. 
 
     .. notes::
         * The default analysis period, if no `start_date` and `end_date` are specified, is determined by the *DEFAULT_ANALYSIS_PERIOD** variable in the `settings,py` file. The default value of this variable is 100.
@@ -501,13 +619,13 @@ def get_daily_price_history(ticker: str, start_date: Union[None, date] = None,
     if cached_prices is not None:
         if asset_type == keys.keys['ASSETS']['EQUITY']:
             logger.debug(
-                f'Comparing {len(cached_prices)} = {dater.business_days_between(start_date, end_date)}')
+                f'Comparing {len(cached_prices)} = {dater.business_days_between(start_date, end_date)}', 'get_daily_price_history')
         elif asset_type == keys.keys['ASSETS']['CRYPTO']:
             logger.debug(
-                f'Comparing {len(cached_prices)} = {dater.days_between(start_date, end_date)}')
+                f'Comparing {len(cached_prices)} = {dater.days_between(start_date, end_date)}', 'get_daily_price_history')
 
     # make sure the length of cache is equal to the length of the requested sample
-    if cached_prices is not None and dater.date_to_string(end_date) in cached_prices.keys() and (
+    if cached_prices is not None and dater.to_string(end_date) in cached_prices.keys() and (
         (asset_type == keys.keys['ASSETS']['EQUITY']
             and (dater.business_days_between(start_date, end_date)) == len(cached_prices))
         or
@@ -519,21 +637,21 @@ def get_daily_price_history(ticker: str, start_date: Union[None, date] = None,
 
     if cached_prices is not None:
         logger.debug(
-            f'Cached {ticker} prices are out of date, passing request off to external service')
+            f'Cached {ticker} prices are out of date, passing request off to external service', 'get_daily_price_history')
 
     prices = price_manager.get_prices(
         ticker=ticker, start_date=start_date, end_date=end_date, asset_type=asset_type)
 
     if cached_prices is not None:
-        new_prices = helper.complement_dict_keys(prices, cached_prices)
+        new_prices = helper.complement_dict_keys(prices, cached_prices)[0]
     else:
         new_prices = prices
 
-    for this_date in new_prices:
-        close_price = new_prices[this_date][keys.keys['PRICES']['OPEN']]
-        open_price = new_prices[this_date][keys.keys['PRICES']['CLOSE']]
-        price_cache.save_row(ticker=ticker, date=this_date,
-                             open_price=open_price, close_price=close_price)
+    price_cache.save_rows(ticker, new_prices)
+
+    if not prices:
+        raise errors.PriceError(
+            f'Prices could not be retrieved for {ticker}')
 
     return prices
 
@@ -549,11 +667,19 @@ def get_daily_price_latest(ticker: str, asset_type: Union[None, str] = None) ->
     2. **asset_type**: str``
         *Optional*. Asset type of the ticker whose history is to be retrieved. Will be calculated from the `ticker` symbol if not provided.
     """
-    prices = get_daily_price_history(ticker=ticker, asset_type=asset_type)
+    last_date = dater.this_date_or_last_trading_date()
+    prices = get_daily_price_history(
+        ticker=ticker, asset_type=asset_type, start_date=last_date, end_date=last_date)
     first_element = helper.get_first_json_key(prices)
     return prices[first_element][keys.keys['PRICES']['OPEN']]
 
 
+def get_daily_prices_latest(tickers: List[str], asset_types: Union[None, List[str]] = None):
+    if asset_types is None:
+        asset_types = [None for _ in tickers]
+    return {ticker: get_daily_price_latest(ticker, asset_types[i]) for i, ticker in enumerate(tickers)}
+
+
 def get_daily_fred_history(symbol: str, start_date: Union[date, None] = None, end_date: Union[date, None] = None) -> list:
     """
     Wrapper around external service request for financial statistics data constructed by the Federal Reserve Economic Data. Relies on an instance of `StatManager` configured by `settings.STAT_MANAGER` value, which in turn is configured by the `STAT_MANAGER` environment variable, to hydrate with data.
@@ -562,9 +688,9 @@ def get_daily_fred_history(symbol: str, start_date: Union[date, None] = None, en
     ----------
     1. **symbol**: ``str`` 
         Symbol representing the statistic whose history is to be retrieved. List of allowable values can be found [here](https://www.quandl.com/data/FRED-Federal-Reserve-Economic-Data/documentation)
-     2. **start_date**: ``datetime.date`` 
+    2. **start_date**: ``Union[date, None]`` 
         *Optional*. Start date of price history. Defaults to None. If `start_date is None`, the calculation is made as if the `start_date` were set to 100 trading days ago. This excludes weekends and holidays.
-    3. **end_date**: ``datetime.date``
+    3. **end_date**: ``Union[date, None]``
         *Optional*. End date of price history. Defaults to None. If `end_date is None`, the calculation is made as if the `end_date` were set to today. This excludes weekends and holidays so that `end_date` is set to the last previous business date.
 
     Returns
@@ -572,6 +698,11 @@ def get_daily_fred_history(symbol: str, start_date: Union[date, None] = None, en
     ``list``: `{ 'date' (str) :  value (str),  'date' (str):  value (str), ... }`
         Dictionary with date strings formatted `YYYY-MM-DD` as keys and the statistic on that date as the corresponding value.
 
+    Raises
+    ------
+    1. **scrilla.errors.PriceError**
+        If no sample prices can be retrieved, this error is thrown. 
+
     .. notes::
         * Most financial statistics are not reported on weekends or holidays, so the `asset_type` for financial statistics is functionally equivalent to equities, at least as far as date calculations are concerned. The dates inputted into this function are validated as if they were labelled as equity `asset_types` for this reason.
 
@@ -583,6 +714,10 @@ def get_daily_fred_history(symbol: str, start_date: Union[date, None] = None, en
     stats = stat_manager.get_stats(
         symbol=symbol, start_date=start_date, end_date=end_date)
 
+    if not stats:
+        raise errors.PriceError(
+            f'Prices could not be retrieved for {symbol}')
+
     return stats
 
 
@@ -626,22 +761,25 @@ def get_daily_interest_history(maturity: str, start_date: Union[date, None] = No
     start_date, end_date = errors.validate_dates(
         start_date=start_date, end_date=end_date, asset_type=keys.keys['ASSETS']['EQUITY'])
 
-    rates = None
     rates = interest_cache.filter_interest_cache(
         maturity, start_date=start_date, end_date=end_date)
 
+    if rates is not None:
+        logger.debug(
+            f'Comparing {len(rates)} = {dater.business_days_between(start_date, end_date, True)}', 'get_daily_interest_history')
+
     # TODO: this only works when stats are reported daily and that the latest date in the dataset is actually end_date.
-    if rates is not None and dater.date_to_string(end_date) in rates.keys() \
-            and (dater.business_days_between(start_date, end_date)) == len(rates):
+    if rates is not None and \
+            dater.to_string(end_date) in rates.keys() and \
+            dater.business_days_between(start_date, end_date) == len(rates):
         return rates
 
     logger.debug(
-        f'Cached {maturity} data is out of date, passing request to external service')
+        f'Cached {maturity} data is out of date, passing request to external service', 'get_daily_interest_history')
     rates = stat_manager.get_interest_rates(
         start_date=start_date, end_date=end_date)
 
-    for this_date in rates:
-        interest_cache.save_row(date=this_date, value=rates[this_date])
+    interest_cache.save_rows(rates)
 
     rates = stat_manager.format_for_maturity(maturity=maturity, results=rates)
 
@@ -657,10 +795,10 @@ def get_daily_interest_latest(maturity: str) -> float:
     1. **maturity**: ``str``
         Maturity of the US Treasury security whose interest rate is to be retrieved. Allowable values accessible through `keys.keys['YIELD_CURVE']
     """
-    end_date = dater.get_last_trading_date()
-    start_date = dater.decrement_date_by_business_days(end_date, 1)
+    end_date = dater.get_last_trading_date(True)
+    start_date = dater.decrement_date_by_business_days(end_date, 1, True)
     interest_history = get_daily_interest_history(
-        maturity=maturity, start_date=start_date, end_date=end_date)
+        maturity, start_date, end_date)
     first_element = helper.get_first_json_key(interest_history)
     return interest_history[first_element]
 
@@ -681,7 +819,8 @@ def get_dividend_history(ticker: str) -> dict:
     ``list`` : `{ 'date' (str) :  amount (str),  'date' (str):  amount (str), ... }`
         Dictionary with date strings formatted `YYYY-MM-DD` as keys and the dividend payment amount on that date as the corresponding value.
     """
-    logger.debug(f'Retrieving {ticker} dividends from service')
+    logger.debug(
+        f'Retrieving {ticker} dividends from service', 'get_dividend_history')
     divs = div_manager.get_dividends(ticker=ticker)
     return divs
 
@@ -691,3 +830,10 @@ def get_risk_free_rate() -> float:
     Returns the risk free rate, defined as the annualized yield on a specific US Treasury duration, as a decimal. The US Treasury yield used as a proxy for the risk free rate is defined in the `settings.py` file and is configured through the RISK_FREE environment variable.
     """
     return get_daily_interest_latest(maturity=settings.RISK_FREE_RATE)/100
+
+
+price_manager = PriceManager(settings.PRICE_MANAGER)
+stat_manager = StatManager(settings.STAT_MANAGER)
+div_manager = DividendManager(settings.DIV_MANAGER)
+price_cache = cache.PriceCache()
+interest_cache = cache.InterestCache()
