@@ -1,3 +1,5 @@
+import datetime
+
 constants = {
     'ONE_TRADING_DAY': {
         'EQUITY': (1/252),
@@ -13,8 +15,10 @@ constants = {
     'SIG_FIGS': 5,
     'LOG_LEVEL': {
         'NONE': 'none',
+        'ERROR': 'error',
         'INFO': 'info',
         'DEBUG': 'debug',
         'VERBOSE': 'verbose'
-    }
+    },
+    'YIELD_START_DATE': datetime.date(1990, 1, 2)
 }
