@@ -139,7 +139,7 @@ FUNC_DICT = {
     "moving_averages": {
         'name': 'Moving Averages Series',
         'values': ["mov-averages", "mas"],
-        'args': ['start_date', 'end_date'],
+        'args': ['start_date', 'end_date', keys.keys['ESTIMATION']['MOMENT'], keys.keys['ESTIMATION']['PERCENT'], keys.keys['ESTIMATION']['LIKE']],
         'description': "Calculate the current moving averages. If no start or end dates are specified, calculations default to the last 100 days of prices.",
         'tickers': True,
     },
@@ -181,7 +181,7 @@ FUNC_DICT = {
     "plot_moving_averages": {
         'name': 'Plot Moving Averages Series',
         'values': ["plot-moving-averages", "plot-mas"],
-        'args': ['start_date', 'end_date', 'save_file'],
+        'args': ['start_date', 'end_date', 'save_file', keys.keys['ESTIMATION']['MOMENT'], keys.keys['ESTIMATION']['PERCENT'], keys.keys['ESTIMATION']['LIKE']],
         'description': "Generates a grouped bar chart of the moving averages for each equity in the supplied list of ticker symbols. If no start or end dates are specified, calculations default to the last 100 days of prices.",
         'tickers': True,
     },
@@ -299,7 +299,7 @@ FUNC_DICT = {
     }
 }
 """
-A dictionary containing configuration information for function arguments. This dictionary is used at various points in the library, such as `scrilla.gui.widgets.functions` and `scrilla.util.helper.format_args`, to generate function interfaces. 
+A dictionary containing configuration information for function arguments. This dictionary is used at various points in the library, such as `scrilla.gui.widgets.functions` and `scrilla.static.formats.format_args`, to generate function interfaces. 
 """
 
 GROUP_DICT = {
@@ -308,6 +308,9 @@ GROUP_DICT = {
 """
 """
 
+ARG_META_DICT = {
+    'groups': ['estimation_method']
+}
 
 # Every argument has four ways of being inputted: short-dash-long, long-dash-long, short-dash-short, long-dash-short
 ARG_DICT = {
