@@ -1,5 +1,5 @@
 import datetime
-from typing import List
+from typing import Dict, List, Union
 
 from scrilla.static import constants, formats, definitions
 
@@ -173,46 +173,32 @@ def spot_price(ticker, this_spot_price):
     scalar_result(f'{ticker} spot price', formatted_price)
 
 
-def model_price(ticker, this_model_price, model):
+def model_price(ticker: str, this_model_price: Union[str, float], model: str) -> None:
     formatted_price = round(float(this_model_price), 2)
     scalar_result(f'{ticker} {str(model).upper()} price', formatted_price)
 
 
-def risk_profile(profiles):
+def risk_profile(profiles: Dict[str, Dict[str, float]]) -> None:
     for key, value in profiles.items():
         title_line(f'{key} Risk Profile')
         for subkey, subvalue in value.items():
             scalar_result(f'{subkey}', f'{subvalue}', currency=False)
 
 
-def moving_average_result(tickers, averages_output, periods, start_date=None, end_date=None):
-    averages, dates = averages_output
-    MA1_prefix, MA2_prefix, MA3_prefix = f'MA({periods[0]})', f'MA({periods[1]})', f'MA({periods[2]})'
-    if start_date is None and end_date is None:
-        for i, item in enumerate(tickers):
-            title = f'{item} Moving Average of Daily Return for {periods[0]}, {periods[1]} & {periods[0]} Days'
-            title_line(title)
+def moving_average_result(ticker: str, averages: Dict[str, Dict[str, float]]) -> None:
+    """
+    Prints the results of `scrilla.analysis.models.geometric.statistics.calculate_moving_averages` or `scrilla.analysis.models.reversion.statistics.calculate_moving_averages` to *stdout*.
 
-            MA1_title, MA2_title, MA3_title = f'{MA1_prefix}_{item}', f'{MA2_prefix}_{item}', f'{MA3_prefix}_{item}'
-            scalar_result(MA1_title, round(averages[i][0], 2))
-            scalar_result(MA2_title, round(averages[i][1], 2))
-            scalar_result(MA3_title, round(averages[i][2], 2))
-    else:
-        for i, item in enumerate(tickers):
-
-            title = f'{item} Moving Average of Daily Return for {periods[0]}, {periods[1]} & {periods[0]} Days'
-            title_line(title)
-
-            MA1_title, MA2_title, MA3_title = f'{MA1_prefix}_{item}', f'{MA2_prefix}_{item}', f'{MA3_prefix}_{item}'
-            for j, item in enumerate(dates):
-                msg_1 = f'{item} : {MA1_title}'
-                scalar_result(msg_1, round(averages[i][0][j], 2))
-            for j, item in enumerate(dates):
-                msg_2 = f'{item} : {MA2_title}'
-                scalar_result(msg_2, round(averages[i][1][j], 2))
-            for j, item in enumerate(dates):
-                msg_3 = f'{item} : {MA3_title}'
-                scalar_result(msg_3, round(averages[i][2][j], 2))
+    Parameters
+    ----------
+    1. **averages**: ``Dict[str, Dict[str,float]]``
+        The dictionary returned from a call to `scrilla.analysis.models.geometric.statistics.calculate_moving_averages` or `scrilla.analysis.models.reversion.statistics.calculate_moving_averages`.
+    """
+    title_line(f'{ticker} Moving Averages')
+    for this_date, average_dict in averages.items():
+        center(this_date)
+        for avg_key, average in average_dict.items():
+            scalar_result(calculation=avg_key, result=average, currency=False)
 
 
 def screen_results(info, model):
@@ -230,13 +216,15 @@ def screen_results(info, model):
 #         optimal result.
 
 
-def optimal_result(portfolio, allocation, investment=None):
+def optimal_result(portfolio, allocation, investment=None, latest_prices=None):
     title_line('Optimal Percentage Allocation')
     portfolio_percent_result(allocation, portfolio.tickers)
 
     if investment is not None:
-        shares = portfolio.calculate_approximate_shares(allocation, investment)
-        total = portfolio.calculate_actual_total(allocation, investment)
+        shares = portfolio.calculate_approximate_shares(
+            allocation, investment, latest_prices)
+        total = portfolio.calculate_actual_total(
+            allocation, investment, latest_prices)
 
         title_line('Optimal Share Allocation')
         portfolio_shares_result(shares, portfolio.tickers)
@@ -250,7 +238,7 @@ def optimal_result(portfolio, allocation, investment=None):
         allocation), currency=False)
 
 
-def efficient_frontier(portfolio, frontier, investment=None):
+def efficient_frontier(portfolio, frontier, investment=None, latest_prices=None):
     title_line('(Annual Return %, Annual Volatility %) Portfolio')
 
     # TODO: edit title to include dates
@@ -269,8 +257,9 @@ def efficient_frontier(portfolio, frontier, investment=None):
 
         if investment is not None:
             shares = portfolio.calculate_approximate_shares(
-                allocation, investment)
-            total = portfolio.calculate_actual_total(allocation, investment)
+                allocation, investment, latest_prices)
+            total = portfolio.calculate_actual_total(
+                allocation, investment, latest_prices)
 
             title_line('Optimal Share Allocation')
             portfolio_shares_result(shares, portfolio.tickers)
@@ -285,7 +274,7 @@ def efficient_frontier(portfolio, frontier, investment=None):
         print('\n')
 
 
-def correlation_matrix(tickers: List[str], correlation_matrix: List[List[float]], display: bool = True):
+def correlation_matrix(tickers: List[str], correl_matrix: List[List[float]], display: bool = True):
     """
     Parameters
     ----------
@@ -302,7 +291,7 @@ def correlation_matrix(tickers: List[str], correlation_matrix: List[List[float]]
     ------
     A correlation matrix string formatted with new lines and spaces.
     """
-    entire_formatted_result, formatted_title = "", ""
+    entire_formatted_result, formatted_subtitle, formatted_title = "", "", ""
 
     line_length, first_symbol_length = 0, 0
     new_line = ""
@@ -327,9 +316,9 @@ def correlation_matrix(tickers: List[str], correlation_matrix: List[List[float]]
                 new_line += " 100.0%"
 
             else:
-                result = correlation_matrix[i][j]
+                result = correl_matrix[i][j]
                 formatted_result = str(
-                    100*result)[:constants.constants['SIG_FIGS']]
+                    100*result)[:(constants.constants['SIG_FIGS']+1)]
                 new_line += f' {formatted_result}%'
 
         entire_formatted_result += new_line + '\n'
@@ -337,16 +326,25 @@ def correlation_matrix(tickers: List[str], correlation_matrix: List[List[float]]
         if i == 0:
             line_length = len(new_line)
 
-    formatted_title += ' '*(formats.formats['INDENT'] + first_symbol_length+1)
-    for symbol in tickers:
-        sym_len = len(symbol)
-        formatted_title += f' {symbol}' + ' '*(7-sym_len)
+    formatted_subtitle += ' ' * \
+        (formats.formats['INDENT'] + first_symbol_length+1)
+    for i, ticker in enumerate(tickers):
+        sym_len = len(ticker)
+        formatted_subtitle += f' {ticker}' + ' '*(7-sym_len)
         # NOTE: seven is number of chars in ' 100.0%'
-    formatted_title += '\n'
+        if i == 0:
+            formatted_title += f'({ticker},'
+        elif i < len(tickers)-1:
+            formatted_title += f'{ticker},'
+        else:
+            formatted_title += f'{ticker}) correlation matrix'
 
-    whole_thing = formatted_title + entire_formatted_result
+    formatted_subtitle += '\n'
+
+    whole_thing = formatted_subtitle + entire_formatted_result
 
     if display:
+        title_line(formatted_title)
         print(f'\n{whole_thing}')
         return
 
@@ -360,22 +358,26 @@ class Logger():
         self.log_level = log_level
 
     # LOGGING FUNCTIONS
-    def comment(self, msg):
+    def comment(self, msg, method, level="INFO"):
         now = datetime.datetime.now()
         dt_string = now.strftime("%d/%m/%Y %H:%M:%S")
-        print(dt_string, ' :', self.location, ' : ', msg)
+        print(dt_string, ' : ', level, ':',
+              f'{self.location}.{method}', ' : ', msg)
 
-    def info(self, msg):
+    def error(self, msg, method):
+        self.comment(msg, method, 'ERROR',)
+
+    def info(self, msg, method):
         if self.log_level in [constants.constants['LOG_LEVEL']['INFO'],
                               constants.constants['LOG_LEVEL']['DEBUG'],
                               constants.constants['LOG_LEVEL']['VERBOSE']]:
-            self.comment(msg)
+            self.comment(msg, method, 'INFO')
 
-    def debug(self, msg):
+    def debug(self, msg, method):
         if self.log_level in [constants.constants['LOG_LEVEL']['DEBUG'],
                               constants.constants['LOG_LEVEL']['VERBOSE']]:
-            self.comment(msg)
+            self.comment(msg, method, 'DEBUG')
 
-    def verbose(self, msg):
+    def verbose(self, msg, method):
         if self.log_level == constants.constants['LOG_LEVEL']['VERBOSE']:
-            self.comment(msg)
+            self.comment(msg, method, 'VERBOSE')
